#include <allegro.h>

#define sx 320
#define sy 200

#include "misc.c"

#define UP              0
#define UPRIGHT         1
#define RIGHT           2
#define DOWNRIGHT       3
#define DOWN            4
#define DOWNLEFT        5
#define LEFT            6
#define UPLEFT          7

#define SPEED   2

#define YES     1
#define NO      0

int ball_x, ball_y, ball_dir, dead;
BITMAP *ball;
BITMAP *paddle;

// if a given point within a given rectangular area?
int MISC_IsWithin( int AreaX1, int AreaY1, int AreaX2, int AreaY2, int ObjX, int ObjY)
{
        if( (ObjX>=AreaX1) && (ObjX<=AreaX2) && (ObjY>=AreaY1) && (ObjY<=AreaY2) )
                return YES;
        return NO;
}

// is any part of a rectangular object within a given rectangular area?
int MISC_ObjectWithin( int AreaX1, int AreaY1, int AreaX2, int AreaY2, int ObjX1, int ObjY1, int ObjX2, int ObjY2)
{
        // are any of the four points of the object within the area?
        if( MISC_IsWithin( AreaX1, AreaY1, AreaX2, AreaY2, ObjX1, ObjY1) )
            return YES; // yes point x1, y1 is
        else
        if( MISC_IsWithin( AreaX1, AreaY1, AreaX2, AreaY2, ObjX2, ObjY1) )
            return YES; // yes point x2, y1 is
        else
        if( MISC_IsWithin( AreaX1, AreaY1, AreaX2, AreaY2, ObjX2, ObjY2) )
            return YES; // yes point x2, y2 is
        else
        if( MISC_IsWithin( AreaX1, AreaY1, AreaX2, AreaY2, ObjX1, ObjY2) )
            return YES; // yes point x1, y1 is

        // is the area within the object ?
        else
            if( MISC_IsWithin( ObjX1, ObjY1, ObjX2, ObjY2, AreaX1, AreaY1) )
                return YES;
        else
            if( MISC_IsWithin( ObjX1, ObjY1, ObjX2, ObjY2, AreaX2, AreaY1) )
                return YES;
        else
            if( MISC_IsWithin( ObjX1, ObjY1, ObjX2, ObjY2, AreaX2, AreaY2) )
                return YES;
        else
            if( MISC_IsWithin( ObjX1, ObjY1, ObjX2, ObjY2, AreaX1, AreaY2) )
                return YES;

        return  NO;
}

void MoveBall( int *x, int *y, int *dir)
{
        int px, py;

        px = *x;
        py = *y;

        switch( *dir)
        {
                case UP: py-=SPEED; break;
                case UPRIGHT: px+=SPEED; py-=SPEED; break;
                case RIGHT: px+=SPEED; break;
                case DOWNRIGHT: px+=SPEED; py+=SPEED; break;
                case DOWN: py+=SPEED; break;
                case DOWNLEFT: px-=SPEED; py+=SPEED; break;
                case LEFT: px-=SPEED; break;
                case UPLEFT: px-=SPEED; py-=SPEED; break;
        }

        if( py +ball->h > SCREEN_H)
            dead =1;

        if( (px < 0) OR (py < 0) OR (px + ball->w > SCREEN_W -1) OR (py + ball->h > SCREEN_H -1) OR MISC_ObjectWithin( px, py, px+ball->w, py+ball->h, mouse_x, mouse_y, mouse_x+paddle->w, mouse_y+paddle->h) )
        {
                KMISC_play_sound( "hit");
                switch( *dir)
                {
                        case UP: *dir=DOWN; break;
                        case UPRIGHT: if(rand()%2) *dir=UPLEFT; else *dir = UPRIGHT; break;
                        case RIGHT: *dir=LEFT; break;
                        case DOWNRIGHT: if(rand()%2) *dir=UPRIGHT; else *dir = UPLEFT; break;
                        case DOWN: *dir=UP; break;
                        case DOWNLEFT: if(rand()%2) *dir=DOWNRIGHT; else *dir = DOWNLEFT; break;
                        case LEFT: *dir=RIGHT; break;
                        case UPLEFT: if(rand()%2) *dir=DOWNLEFT; else *dir = DOWNRIGHT; break;
                }
        }
        else
        {
                *x = px;
                *y = py;
        }


}

int main()
{
        BITMAP *DoubleBuffer, *GameOver;

        KMISC_init();
        KMISC_set_datafile_name("paddle.dat");
        KMISC_init_graphics();
        install_mouse();
        DoubleBuffer = create_bitmap( SCREEN_W, SCREEN_H);

        paddle = KMISC_load_data_object("paddle");
        ball = KMISC_load_data_object("ball");

        set_mouse_sprite( paddle);
        set_mouse_range( 0, SCREEN_H - paddle->h-2, SCREEN_W - paddle->w -1, SCREEN_H - paddle->h-2);

        ball_x = 1;
        ball_y = 1;
        ball_dir = DOWNRIGHT;
        dead = 0;

        KMISC_init_sound();

        install_keyboard();

        while( (!key[ KEY_ESC] ) && (!dead) )
        {
                clear_to_color(DoubleBuffer,125);
                show_mouse( DoubleBuffer);

                MoveBall( &ball_x, &ball_y, &ball_dir);
                draw_sprite( DoubleBuffer, ball, ball_x, ball_y);

                if( key[ KEY_LEFT])
                {
                        if( mouse_x > 0)
                            position_mouse( mouse_x -6, mouse_y);
                }
                else
                if( key[ KEY_RIGHT])
                        if( mouse_x < SCREEN_W - paddle->w)
                            position_mouse( mouse_x +6, mouse_y);

                vsync();
                blit( DoubleBuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);


        }
        remove_keyboard();

        if( dead)
        {
                GameOver = KMISC_load_data_object("gameover");
                draw_sprite( screen, GameOver, (SCREEN_W - GameOver->w)/2, (SCREEN_H - GameOver->h)/2 );

                getch();
                destroy_bitmap( GameOver);
        }

        KMISC_shutdown_sound();
        set_mouse_sprite( NULL);
        destroy_bitmap( DoubleBuffer);
        destroy_bitmap( paddle);
        destroy_bitmap( ball);
        return 0;
}

