#include <string.h>
#include <stdarg.h>

// colours
#define CL_NONE         0
#define CL_BLACK        3
#define CL_RED          4
#define CL_GREEN        2
#define CL_BLUE         1
#define CL_YELLOW       14
#define CL_WHITE        15

#define OR ||
// vars
char KMISC_DataFileName[128] = "mydata.dat";
SAMPLE *KMISC_MySample;

// prototypes
void KMISC_init();
void KMISC_error( char *str, ...);
void *KMISC_load_data_object( char *ObjectName);
void KMISC_init_pallette();
void KMISC_set_datafile_name( char *filename);
void KMISC_init_sound();
void KMISC_shutdown_sound();
void KMISC_init_graphics();
void KMISC_shutdown_graphics();
int KMISC_play_sound( char *EntryName);

// code
void KMISC_init()
{
        allegro_init();
        install_timer();
}

void KMISC_error( char *str, ...)
{
	char str2[128];
	va_list args;

	va_start(args,str);
	vsprintf(str2,str,args);
	va_end(args);

        KMISC_shutdown_sound();
        KMISC_shutdown_graphics();

        printf("Error - %s!\n", str2);
        exit(1);
}

void *KMISC_load_data_object( char *ObjectName)
{
        DATAFILE *MyData;
        MyData = load_datafile_object( KMISC_DataFileName, ObjectName);
        if( MyData == NULL)
                KMISC_error("LoadDataObject: Entry not found \"%s\" in \"%s\"", ObjectName, KMISC_DataFileName);

        return MyData[0].dat;
}

void KMISC_init_pallette()
{
        RGB *pal;

        pal = KMISC_load_data_object( "_palette");

        pal[0].r = 0;
        pal[0].g = 0;
        pal[0].b = 0;

        set_palette( pal);

}

void KMISC_set_datafile_name( char *filename)
{
        strcpy( KMISC_DataFileName, filename);
}

void KMISC_init_sound()
{
        detect_digi_driver(DIGI_AUTODETECT);
        detect_midi_driver(MIDI_AUTODETECT);

        set_volume( 255, 255);

        if( install_sound(DIGI_AUTODETECT,MIDI_AUTODETECT, "") )
        {
                KMISC_error("KMISC_init_install_sound() failed");
        }
}

void KMISC_shutdown_sound()
{
        destroy_sample( KMISC_MySample);
        remove_sound();
}

void KMISC_init_graphics()
{
        set_color_depth( 8);
        if( set_gfx_mode( GFX_AUTODETECT, sx, sy, sx, sy) < 0)
            KMISC_error("KMISC_init_graphics: Unable to obtain %d by %d graphics", sx, sy);

        KMISC_init_pallette();
}

void KMISC_shutdown_graphics()
{
        set_gfx_mode( GFX_TEXT, sx, sy, sx, sy);
}

int KMISC_play_sound( char *EntryName)
{
        KMISC_MySample = KMISC_load_data_object( EntryName);
        play_sample( KMISC_MySample, 255, 128, 1000, FALSE);
}

